@@ -1,4 +1,4 @@
-/* Copyright (c) 2003 SuSE Linux AG, Germany
+/* Copyright (c) 2003, 2004 SuSE Linux AG, Germany
  * Copyright (c) 1999, 2000, 2001, 2002 SuSE GmbH Nuernberg, Germany
  * Author: Thorsten Kukuk <kukuk@suse.de>
  *
@@ -97,7 +97,7 @@
 #endif
 
 #ifdef WITH_SELINUX
-int selinux_enabled=0;
+int selinux_enabled = -1;
 security_context_t user_context=NULL;
 security_context_t ttyn_context=NULL; /* The current context of ttyn device */
 security_context_t vcsn_context=NULL; /* The current context of vcsn device */
@@ -529,7 +529,7 @@ main (int argc, char **argv)
   char *buffer = alloca (buflen);
   struct passwd resultbuf;
   struct passwd *pwd;
-  char vcsn[20], vcsan[20];
+  char *vcsn = NULL, *vcsan = NULL;
 
 
   init_sighandler ();
@@ -922,14 +922,17 @@ main (int argc, char **argv)
 
 #ifdef WITH_SELINUX
   /* Make sure SELINUX is really running on this system */
-  if ((selinux_enabled = is_selinux_enabled ()))
+  selinux_enabled = is_selinux_enabled ();
+  if (selinux_enabled > 0)
     {
       int ret;
       security_context_t* contextlist;
       int num_contexts = get_ordered_context_list (username, 0, &contextlist);
       if (num_contexts > 0)
 	{
-#if 0
+#if 1
+	  /* Allow the user to select the context, if more than
+	     one exist.  */
 	  ret = query_user_context (contextlist, &user_context);
 	  freeconary (contextlist);
 
@@ -941,6 +944,8 @@ main (int argc, char **argv)
 	      exit (3);
 	    }
 #else
+	  /* Always use the default context. User should use "newrole"
+	     to change the context.  */
 	  user_context = (security_context_t) strdup (contextlist[0]);
 	  freeconary (contextlist);
 #endif
@@ -1119,7 +1124,7 @@ main (int argc, char **argv)
   chmod (ttyn,  getdef_num ("TTYPERM", 0600));
 
 #ifdef WITH_SELINUX
-  if (selinux_enabled)
+  if (selinux_enabled > 0)
     ttyn_context = security_label_tty (ttyn, user_context);
 #endif
 
@@ -1133,10 +1138,10 @@ main (int argc, char **argv)
       while (*p && !isdigit (*p))
 	++p;
 
-      strcpy (vcsn, "/dev/vcs");
-      strcat (vcsn, p);
-      strcpy (vcsan, "/dev/vcsa");
-      strcat (vcsan, p);
+      if (asprintf (&vcsn, "/dev/vcs%s", p) < 0)
+	vcsn = NULL;
+      if (asprintf (&vcsan, "/dev/vcsa%s", p) < 0)
+	vcsan = NULL;
 #if 0
       /*
        * Please don't add code to chown /dev/vcs* to the user logging in -
@@ -1145,16 +1150,24 @@ main (int argc, char **argv)
        * have the *BSD revoke() system call yet, and vhangup() only works
        * for tty devices (which vcs* is not).
        */
-      chown (vcsn, pwd->pw_uid, gid);
-      chown (vcsan, pwd->pw_uid, gid);
-      chmod (vcsn, getdef_num ("TTYPERM", 0600));
-      chmod (vcsan, getdef_num ("TTYPERM", 0600));
+      if (vcsn)
+	{
+	  chown (vcsn, pwd->pw_uid, gid);
+	  chmod (vcsn, getdef_num ("TTYPERM", 0600));
+	}
+      if (vcsan)
+	{
+	  chown (vcsan, pwd->pw_uid, gid);
+	  chmod (vcsan, getdef_num ("TTYPERM", 0600));
+	}
 #endif
 #ifdef WITH_SELINUX
-      if (selinux_enabled)
+      if (selinux_enabled > 0)
         {
-	  vcsn_context = security_label_tty (vcsn, user_context);
-	  vcsan_context = security_label_tty (vcsan, user_context);
+	  if (vcsn)
+	    vcsn_context = security_label_tty (vcsn, user_context);
+	  if (vcsan)
+	    vcsan_context = security_label_tty (vcsan, user_context);
 	}
 #endif
     }
@@ -1238,7 +1251,7 @@ main (int argc, char **argv)
   setproctitle ("login", username);
 
 #ifdef WITH_SELINUX
-  if (selinux_enabled)
+  if (selinux_enabled > 0)
     {
       if (pwd->pw_uid == 0)
 	{
@@ -1309,18 +1322,24 @@ main (int argc, char **argv)
 		     childPid, strerror (errsv));
 	}
 #ifdef WITH_SELINUX
-      if (selinux_enabled)
+      if (selinux_enabled > 0)
         {
           /* We need to change the contexts of the terminal devices back to
              the system when the user's session ends.  */
           security_restorelabel_tty (ttyn, ttyn_context);
           if (consoletty (0))
 	    {
-	      security_restorelabel_tty (vcsn, vcsn_context);
-	      security_restorelabel_tty (vcsan, vcsan_context);
+	      if (vcsn)
+		security_restorelabel_tty (vcsn, vcsn_context);
+	      if (vcsan)
+		security_restorelabel_tty (vcsan, vcsan_context);
 	    }
         }
 #endif
+      if (vcsn)
+	free (vcsn);
+      if (vcsan)
+	free (vcsan);
 
       PAM_END;
       exit (0);
@@ -1341,7 +1360,7 @@ main (int argc, char **argv)
   openlog ("login", LOG_ODELAY | LOG_PID, LOG_AUTHPRIV);
 
 #ifdef WITH_SELINUX
-  if (selinux_enabled)
+  if (selinux_enabled > 0)
     {
       if (setexeccon (user_context))
 	{
